/*
 * misc.h
 *
 * Created: 04.12.2017 16:50:49
 *  Author: Thomas F
 */ 


#ifndef MISC_H_
#define MISC_H_



#define OPTIMIZATION_BARRIER __ASM volatile ("dmb" : : : "memory");


// ########## Function

#define RAM_FUNC __attribute__ ((long_call, section (".ramfunc")))
#define FLASH_FUNC
#define INLINE_FUNC static inline __attribute__((always_inline))


// ########## Sections

#define RAM __attribute__ ((section(".sram")))
#define FLASH __attribute__ ((section(".flash")))


// ########## IRQ

#define ISR(func) void func(void)	// Cortex Interrupt handlers can be programmed as normal C functions 

INLINE_FUNC void EnableIRQ(void)
{
	__DMB();
	__enable_irq();
}

INLINE_FUNC void DisableIRQ(void)
{
	__disable_irq();
	__DMB();
}


// ########## PORT

#define PORTA           ((PortGroup*)&PORT->Group[0])
#define PORTB           ((PortGroup*)&PORT->Group[1])
//#define PORTC           ((PortGroup*)&PORT->Group[2])

#define PMUX_A	0
#define PMUX_B	1
#define PMUX_C	2
#define PMUX_D	3
#define PMUX_E	4
#define PMUX_F	5
#define PMUX_G	6
#define PMUX_H	7
#define PMUX_I	8
#define PMUX_J	9
#define PMUX_K	10
#define PMUX_L	11
#define PMUX_M	12
#define PMUX_N	13


// ########## Delay s / ms

INLINE_FUNC void Delay_us(uint32_t delay)
{
	uint32_t UsValue = ((SystemCoreClock / 1000000) * delay) - 70;

	CMCC->MCFG.reg = CMCC_MCFG_MODE_CYCLE_COUNT;
	CMCC->MEN.reg = CMCC_MEN_MENABLE;
	CMCC->MCTRL.reg = CMCC_MCTRL_SWRST;
	
	while(CMCC->MSR.reg < UsValue) { ; }
}

INLINE_FUNC void Delay_ms(uint32_t delay)
{
	uint32_t UsValue = ((SystemCoreClock / 1000) * delay) - 70;
	
	CMCC->MCFG.reg = CMCC_MCFG_MODE_CYCLE_COUNT;
	CMCC->MEN.reg = CMCC_MEN_MENABLE;
	CMCC->MCTRL.reg = CMCC_MCTRL_SWRST;
	
	while(CMCC->MSR.reg < UsValue) { ; }
}


#endif /* MISC_H_ */